==> maxdb-all-linux-64bit-x86_64-7_6_00_36.tgz: SAPDB/Install/Window/LogFileViewer.pm <==
#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_36/sys/src/install/perl/SAPDB/Install/Window/LogFileViewer.pm#1 $
# $DateTime: 2006/11/17 17:24:42 $
# $Change: 140017 $
#
# Desc: module to view installer logfiles with syntax highlighting, filter and search function


package SAPDB::Install::Window::LogFileViewer;

use Wx qw(	wxVERTICAL wxEXPAND wxALL
			wxTE_RICH2 wxTE_MULTILINE
			wxTE_READONLY wxRED wxBLUE
			wxGREEN wxBLACK wxCYAN 
			wxLIGHT_GREY wxTOP wxWHITE
			wxBLACK wxHSCROLL wxTE_NOHIDESEL );

use base Wx::Panel;
use SAPDB::Install::Window::FindBar;


our $labels = {'ERR: ' => {
							'desc' => 'error',
							'colour' => wxRED,
							'regex' => 'error',
							'REGEX' => 'ERR'
							},
				'WRN: ' => {
							'desc' => 'warning',
							'colour' => Wx::Colour->new(255,165,0),
							'regex' => 'warning|failed'	
						  },
				'STDOUT: ' => {
							'desc' => 'stdout output',
							'colour' => wxLIGHT_GREY	
						  },
				'STDERR: ' => {
							'desc' => 'stderr output',
							'colour' => wxRED	
						  },
				'MSG: ' => {
							'desc' => 'log message',
							'colour' => wxBLACK
						},
				'SYS: ' => {
							'desc' => 'creating child processes',
							'colour' => wxBLUE
						},
				'CALL: ' => {
							'desc' => 'command line arguments',
							'colour' => wxGREEN
						}
			 };
			 
$labels->{'ACTION: '} = $labels->{'MSG: '};			 
our $default_label = 'MSG: ';

sub new{
	my ($self ) = shift->SUPER::new(@_);
	
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	
	$self->{'txt_ctrl'} = Wx::TextCtrl->new($self,
									-1,'',[-1,-1],[50,50],
									wxTE_RICH2 | wxTE_MULTILINE |
									wxTE_READONLY | wxHSCROLL | wxTE_NOHIDESEL);
	
	$sizer->Add($self->{'txt_ctrl'},1,wxEXPAND | wxALL,5);
	
	$self->{'find_bar'} = SAPDB::Install::Window::FindBar->new($self,-1);
	$sizer->Add($self->{'find_bar'},0,wxEXPAND | ((wxALL) & (~wxTOP)) ,5);
	EVT_SDB_FIND_NEXT ($self,$self->{'find_bar'}->GetId,\&OnFindNext);
	EVT_SDB_FIND_PREVIOUS ($self,$self->{'find_bar'}->GetId,\&OnFindPrevious);
	EVT_SDB_FIND_MATCH_CASE ($self,$self->{'find_bar'}->GetId,\&OnMatchCase);
	EVT_SDB_FIND_TEXT ($self,$self->{'find_bar'}->GetId,\&OnFindText);
	$self->SetSizer($sizer);
	$self->SetAutoLayout(1);
	return $self;
}

sub XYToPosition{
        my ($self,$x,$y) = @_;
        my $pos = 0;
        for (my $i = 0; $i < $y;$i++){
                $pos += length ($self->{'lines'}->[$i]->{'value'}) + 1;
        }
        $pos += $x;
        return $pos;
}



sub SetValue{
	my ($self,$value) = @_;
	
	my $current_label;
	my @lines;
			
	$self->{'value'} = $value;
	
	my $tmp;
	
	foreach my $line (split("\n",$value)){
		my ($label) =  ($line =~ /^(\S+:\s)/);
		if (defined $label and exists $labels->{$label}){
			#$line =~ s/^\S+:\s//;
		}
		else{
			my $found_label;
			foreach my $i (keys(%$labels)){
	
				if (exists $labels->{$i}->{'REGEX'}){
					my $pattern = $labels->{$i}->{'REGEX'};
					if ($line =~ /$pattern/){
						$found_label = $i;
						last;
					}
				}  
	
				if (exists $labels->{$i}->{'regex'}){
					my $pattern = $labels->{$i}->{'regex'};
					if ($line =~ /$pattern/i){
						$found_label = $i;
						last;
					}
				}
			}
			$label = defined $found_label ? $found_label : $default_label;
		}
		
		if ($current_label ne $label){
			$current_label = $label;
		}
		push @lines , {'value' => $line,
					   'colour' => $labels->{$label}->{'colour'}};
		$tmp .= $line."\n";			   
	}

	$self->{'txt_ctrl'}->SetValue($tmp);
	$self->{'lines'} = \@lines;	
	
	my $line_start = 0;
	my $current_color = 0;
	my $current_start = 0;
	
	foreach my $line (@lines){
		if ($current_color != $line->{'colour'}){
			if($current_color != 0){
				$self->{'txt_ctrl'}->SetStyle(	$current_start,
												$line_start - 1,
												Wx::TextAttr->new($current_color));	
			}
			$current_start = $line_start; 			
			$current_color = $line->{'colour'};
				
		}
		$line_start += length($line->{'value'}) + 1;
	}
	
	if ($current_color != 0 and ($line_start - 1) > $current_start){
		$self->{'txt_ctrl'}->SetStyle(	$current_start,
										$line_start - 1,
										Wx::TextAttr->new($current_color));
	}
	
	$self->{'txt_ctrl'}->ShowPosition (0);
	return 1;
}

sub FindFirst{
	my ($self,$string) = @_;
	$self->{'search_string'} = $string;
	my $pattern = $string; 
	if (defined $string){
		foreach my $char (qw ( \ . + ? / * [ ] ( ) )){
			my $s = '\\'.$char;
			my $d = '\\'.$char;
			$pattern =~ s/$s/$d/g;
		}
		$self->{'search_pattern'} = $pattern;
	}
		
	return 1 if $self->{'search_string'} eq '';
	
	my $line_num = 0;
	
	foreach my $line (@{$self->{'lines'}}){
		pos ($line->{'value'}) = 0;
		if ($self->{'match_case'} and $line->{'value'} =~ m/$pattern/g or 
		    !$self->{'match_case'} and $line->{'value'} =~ m/$pattern/gi){
			my $position = pos ($line->{'value'});
					
			my $stop = $^O =~ /mswin/i ? 
					$self->{'txt_ctrl'}->XYToPosition($position,$line_num) :
					$self->XYToPosition($position,$line_num);
			my $start = $stop - length ($self->{'search_string'});
			$self->{'txt_ctrl'}->SetSelection($start,$stop);
			$self->{'txt_ctrl'}->ShowPosition($start) if ($^O !~ /mswin/i);	
			last;				
		}
		$line_num++;
	}
	
	
		
}

sub FindNext{
	my ($self) = @_;
	if (defined $self->{'search_pattern'}){
		
		my $pattern = $self->{'search_pattern'};
    	my ($x,$current_line) = $self->{'txt_ctrl'}->PositionToXY(($self->{'txt_ctrl'}->GetSelection ())[0] + 1);
		
		pos ($self->{'lines'}->[$current_line]->{'value'}) = $x;
		
		foreach my $i ($current_line .. $#{@{$self->{'lines'}}}){
			my $line = $self->{'lines'}->[$i];
			if ($self->{'match_case'} and $line->{'value'} =~ m/$pattern/g or 
				!$self->{'match_case'} and $line->{'value'} =~ m/$pattern/gi){
				my $position = pos ($line->{'value'});
				my $stop = $^O =~ /mswin/i ?
                                        $self->{'txt_ctrl'}->XYToPosition($position,$i) :
                                        $self->XYToPosition($position,$i);
			    my $start = $stop - length ($self->{'search_string'});
				$self->{'txt_ctrl'}->SetSelection($start,$stop);
				$self->{'txt_ctrl'}->ShowPosition($start) if ($^O !~ /mswin/i);	
				last;				
			}
		} 
	}	
}

sub FindPrevious{
	my ($self) = @_;
	if (defined $self->{'search_pattern'}){
		my $pattern = $self->{'search_pattern'};
    	
    	my @position = $self->{'txt_ctrl'}->GetSelection ();
    	
    	
    	my ($line_pos,$current_line) = $self->{'txt_ctrl'}->PositionToXY($position[0]);
		
		pos ($self->{'lines'}->[$current_line]->{'value'}) = 0;
		
		my $i = $current_line;		
		
		while ($i > -1){
			my $line = $self->{'lines'}->[$i];
			pos ($line->{'value'}) = 0;
			my @result = ();
			my $stop;
			
			while ($self->{'match_case'} and $line->{'value'} =~ m/$pattern/g or 
				!$self->{'match_case'} and $line->{'value'} =~ m/$pattern/gi){
				unshift @result, pos ($line->{'value'});			
			}
			
			if ($#result == -1){
				$i--;
				next;
			}
				
			
			if ($i == $current_line){
				my $found = 0;
				foreach my $pos (@result){
					if ($pos < $line_pos){
						$stop = $^O =~ /mswin/i ?
		                                        $self->{'txt_ctrl'}->XYToPosition($pos,$i) :
                		                        $self->XYToPosition($pos,$i);
						$found = 1;
						last;
					}
				}
				unless ($found){
					$i--;
					next;
				}
			}
			else{
				$stop = $^O =~ /mswin/i ?
                                        $self->{'txt_ctrl'}->XYToPosition($result[0],$i) :
                                        $self->XYToPosition($result[0],$i);
			}
			
			my $start = $stop - length ($self->{'search_string'});
			$self->{'txt_ctrl'}->SetSelection($start,$stop);
			$self->{'txt_ctrl'}->ShowPosition($start) if ($^O !~ /mswin/i);
			last;				
		} 
	}	
}

sub OnFindNext{
	my ($self,$evt) = @_;
	$self->FindNext ();
}


sub OnFindPrevious{
	my ($self,$evt) = @_;
	$self->FindPrevious ();
}


sub OnMatchCase{
	my ($self,$evt) = @_;
	$self->{'match_case'} = $evt->IsChecked ();
	unless ($self->{'match_case'}){
		return 1;
	}
	my $text = $self->{'txt_ctrl'}->GetStringSelection();
	
	my $pattern = $self->{'search_pattern'};
	
	if ($text !~ /$pattern/){
		$self->FindNext();
	}
	return 1;
}


sub ResetSelections{
	my ($self) = @_;
	unless (defined $self->{'selection'}){
		return 1;
	}
	my ($start,$stop) = @{$self->{'selection'}};
	my ($x,$line) = $self->{'txt_ctrl'}->PositionToXY($start);
	$self->{'txt_ctrl'}->SetStyle($start,$stop,
			Wx::TextAttr->new($self->{'lines'}->[$line]->{'colour'},wxWHITE));
	$self->{'selection'} = undef;
	return 1;
}


sub OnFindText{
	my ($self,$evt) = @_;
	$self->FindFirst($evt->GetString);
}

sub GetValue{
	my ($self) = @_;
	return $self->{'value'};
}


sub Destroy{
	my ($self) = @_;
	while($self->GetSizer->Remove(0)){}
	$self->{'txt_ctrl'}->Destroy();
	$self->{'find_bar'}->Destroy();
	$self->SUPER::Destroy;
}
1;

==> maxdb-all-linux-64bit-x86_64-7_6_03_15.tgz: SAPDB/Install/Window/LogFileViewer.pm <==
#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_03/b_15/sys/src/install/perl/SAPDB/Install/Window/LogFileViewer.pm#1 $
# $DateTime: 2008/01/18 16:05:39 $
# $Change: 173096 $
#
# Desc: module to view installer logfiles with syntax highlighting, filter and search function


package SAPDB::Install::Window::LogFileViewer;

use Wx qw(	wxVERTICAL wxEXPAND wxALL
			wxTE_RICH2 wxTE_MULTILINE
			wxTE_READONLY wxRED wxBLUE
			wxGREEN wxBLACK wxCYAN 
			wxLIGHT_GREY wxTOP wxWHITE
			wxBLACK wxHSCROLL wxTE_NOHIDESEL );

use base Wx::Panel;
use SAPDB::Install::Window::FindBar;

#
# Using SetDefaultStyle() + AppendText() for highlighting text for wxMSW
# because of lesser memory usage.
# wxGTK based on Gtk+ 1.x (currently AIX, HP-UX and Tru64) goes into an 
# endless loop while using SetValue() + SetStyle(). Because of this issue using
# SetDefaultStyle() + AppendText() here too. 
#
our $USE_SETDEFAULTSTYLE = ($^O =~ /mswin|hpux|aix|osf/i);


our $labels = {'ERR: ' => {
							'desc' => 'error',
							'colour' => wxRED,
							'regex' => 'error[:\s]',
							'REGEX' => '\sERR'
							},
				'WRN: ' => {
							'desc' => 'warning',
							'colour' => Wx::Colour->new(255,165,0),
							'regex' => 'warning[\s:]|failed'	
						  },
				'STDOUT: ' => {
							'desc' => 'stdout output',
							'colour' => wxLIGHT_GREY	
						  },
				'STDERR: ' => {
							'desc' => 'stderr output',
							'colour' => wxRED	
						  },
				'MSG: ' => {
							'desc' => 'log message',
							'colour' => wxBLACK
						},
				'SYS: ' => {
							'desc' => 'creating child processes',
							'colour' => wxBLUE
						},
				'CALL: ' => {
							'desc' => 'command line arguments',
							'colour' => wxGREEN
						}
			 };
			 
$labels->{'ACTION: '} = $labels->{'MSG: '};			 
our $default_label = 'MSG: ';

sub new{
	my ($self ) = shift->SUPER::new(@_);
	
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	
	$self->{'txt_ctrl'} = Wx::TextCtrl->new($self,
									-1,'',[-1,-1],[50,50],
									wxTE_RICH2 | wxTE_MULTILINE |
									wxTE_READONLY | wxHSCROLL | wxTE_NOHIDESEL);
	
	$sizer->Add($self->{'txt_ctrl'},1,wxEXPAND | wxALL,5);
	
	$self->{'find_bar'} = SAPDB::Install::Window::FindBar->new($self,-1);
	$sizer->Add($self->{'find_bar'},0,wxEXPAND | ((wxALL) & (~wxTOP)) ,5);
	EVT_SDB_FIND_NEXT ($self,$self->{'find_bar'}->GetId,\&OnFindNext);
	EVT_SDB_FIND_PREVIOUS ($self,$self->{'find_bar'}->GetId,\&OnFindPrevious);
	EVT_SDB_FIND_MATCH_CASE ($self,$self->{'find_bar'}->GetId,\&OnMatchCase);
	EVT_SDB_FIND_TEXT ($self,$self->{'find_bar'}->GetId,\&OnFindText);
	$self->SetSizer($sizer);
	$self->SetAutoLayout(1);
	return $self;
}

sub XYToPosition{
        my ($self,$x,$y) = @_;
        my $pos = 0;
        for (my $i = 0; $i < $y;$i++){
                $pos += length ($self->{'lines'}->[$i]->{'value'}) + 1;
        }
        $pos += $x;
        return $pos;
}



sub SetValue{
	my ($self,$value) = @_;
	
	my $current_label;
	my @lines;
			
	$self->{'value'} = $value;
	
	my $tmp;
	
	foreach my $line (split("\n",$value)){
		my ($label) =  ($line =~ /^(\S+:\s)/);
		if (defined $label and exists $labels->{$label}){
			#$line =~ s/^\S+:\s//;
		}
		else{
			my $found_label;
			foreach my $i (keys(%$labels)){
	
				if (exists $labels->{$i}->{'REGEX'}){
					my $pattern = $labels->{$i}->{'REGEX'};
					if ($line =~ /$pattern/){
						$found_label = $i;
						last;
					}
				}  
	
				if (exists $labels->{$i}->{'regex'}){
					my $pattern = $labels->{$i}->{'regex'};
					if ($line =~ /$pattern/i){
						$found_label = $i;
						last;
					}
				}
			}
			$label = defined $found_label ? $found_label : $default_label;
		}
		
		if ($current_label ne $label){
			$current_label = $label;
		}
		push @lines , {'value' => $line,
					   'colour' => $labels->{$label}->{'colour'}};
		
		if ($USE_SETDEFAULTSTYLE){
			$self->{'txt_ctrl'}->SetDefaultStyle (Wx::TextAttr->new($labels->{$label}->{'colour'}));
			$self->{'txt_ctrl'}->AppendText ($line."\n");		
		}
		else{
			$tmp .= $line."\n";
		}
		
	}
	
	$self->{'lines'} = \@lines;
	
	if ($USE_SETDEFAULTSTYLE){
		$self->{'txt_ctrl'}->ShowPosition (0);
		return 1;
	}
	
	$self->{'txt_ctrl'}->SetValue($tmp);
		
	my $line_start = 0;
	my $current_color = 0;
	my $current_start = 0;
	
	foreach my $line (@lines){
		if ($current_color != $line->{'colour'}){
			if($current_color != 0){
				$self->{'txt_ctrl'}->SetStyle(	$current_start,
												$line_start - 1,
												Wx::TextAttr->new ($current_color));
			}
			$current_start = $line_start; 			
			$current_color = $line->{'colour'};
				
		}
		$line_start += length($line->{'value'}) + 1;
	}
	
	if ($current_color != 0 and ($line_start - 1) > $current_start){
		$self->{'txt_ctrl'}->SetStyle(	$current_start,
										$line_start - 1,
										Wx::TextAttr->new ($current_color));
	}
	
	$self->{'txt_ctrl'}->ShowPosition (0);
	return 1;
}

sub FindFirst{
	my ($self,$string) = @_;
	$self->{'search_string'} = $string;
	my $pattern = $string; 
	if (defined $string){
		foreach my $char (qw ( \ . + ? / * [ ] ( ) )){
			my $s = '\\'.$char;
			my $d = '\\'.$char;
			$pattern =~ s/$s/$d/g;
		}
		$self->{'search_pattern'} = $pattern;
	}
		
	return 1 if $self->{'search_string'} eq '';
	
	my $line_num = 0;
	
	foreach my $line (@{$self->{'lines'}}){
		pos ($line->{'value'}) = 0;
		if ($self->{'match_case'} and $line->{'value'} =~ m/$pattern/g or 
		    !$self->{'match_case'} and $line->{'value'} =~ m/$pattern/gi){
			my $position = pos ($line->{'value'});
					
			my $stop = $^O =~ /mswin/i ? 
					$self->{'txt_ctrl'}->XYToPosition($position,$line_num) :
					$self->XYToPosition($position,$line_num);
			my $start = $stop - length ($self->{'search_string'});
			$self->{'txt_ctrl'}->SetSelection($start,$stop);
			$self->{'txt_ctrl'}->ShowPosition($start) if ($^O !~ /mswin/i);	
			last;				
		}
		$line_num++;
	}
	
	
		
}

sub FindNext{
	my ($self) = @_;
	if (defined $self->{'search_pattern'}){
		
		my $pattern = $self->{'search_pattern'};
    	my ($x,$current_line) = $self->{'txt_ctrl'}->PositionToXY(($self->{'txt_ctrl'}->GetSelection ())[0] + 1);
		
		pos ($self->{'lines'}->[$current_line]->{'value'}) = $x;
		
		foreach my $i ($current_line .. $#{@{$self->{'lines'}}}){
			my $line = $self->{'lines'}->[$i];
			if ($self->{'match_case'} and $line->{'value'} =~ m/$pattern/g or 
				!$self->{'match_case'} and $line->{'value'} =~ m/$pattern/gi){
				my $position = pos ($line->{'value'});
				my $stop = $^O =~ /mswin/i ?
                                        $self->{'txt_ctrl'}->XYToPosition($position,$i) :
                                        $self->XYToPosition($position,$i);
			    my $start = $stop - length ($self->{'search_string'});
				$self->{'txt_ctrl'}->SetSelection($start,$stop);
				$self->{'txt_ctrl'}->ShowPosition($start) if ($^O !~ /mswin/i);	
				last;				
			}
		} 
	}	
}

sub FindPrevious{
	my ($self) = @_;
	if (defined $self->{'search_pattern'}){
		my $pattern = $self->{'search_pattern'};
    	
    	my @position = $self->{'txt_ctrl'}->GetSelection ();
    	
    	
    	my ($line_pos,$current_line) = $self->{'txt_ctrl'}->PositionToXY($position[0]);
		
		pos ($self->{'lines'}->[$current_line]->{'value'}) = 0;
		
		my $i = $current_line;		
		
		while ($i > -1){
			my $line = $self->{'lines'}->[$i];
			pos ($line->{'value'}) = 0;
			my @result = ();
			my $stop;
			
			while ($self->{'match_case'} and $line->{'value'} =~ m/$pattern/g or 
				!$self->{'match_case'} and $line->{'value'} =~ m/$pattern/gi){
				unshift @result, pos ($line->{'value'});			
			}
			
			if ($#result == -1){
				$i--;
				next;
			}
				
			
			if ($i == $current_line){
				my $found = 0;
				foreach my $pos (@result){
					if ($pos < $line_pos){
						$stop = $^O =~ /mswin/i ?
		                                        $self->{'txt_ctrl'}->XYToPosition($pos,$i) :
                		                        $self->XYToPosition($pos,$i);
						$found = 1;
						last;
					}
				}
				unless ($found){
					$i--;
					next;
				}
			}
			else{
				$stop = $^O =~ /mswin/i ?
                                        $self->{'txt_ctrl'}->XYToPosition($result[0],$i) :
                                        $self->XYToPosition($result[0],$i);
			}
			
			my $start = $stop - length ($self->{'search_string'});
			$self->{'txt_ctrl'}->SetSelection($start,$stop);
			$self->{'txt_ctrl'}->ShowPosition($start) if ($^O !~ /mswin/i);
			last;				
		} 
	}	
}

sub OnFindNext{
	my ($self,$evt) = @_;
	$self->FindNext ();
}


sub OnFindPrevious{
	my ($self,$evt) = @_;
	$self->FindPrevious ();
}


sub OnMatchCase{
	my ($self,$evt) = @_;
	$self->{'match_case'} = $evt->IsChecked ();
	unless ($self->{'match_case'}){
		return 1;
	}
	my $text = $self->{'txt_ctrl'}->GetStringSelection();
	
	my $pattern = $self->{'search_pattern'};
	
	if ($text !~ /$pattern/){
		$self->FindNext();
	}
	return 1;
}


sub ResetSelections{
	my ($self) = @_;
	unless (defined $self->{'selection'}){
		return 1;
	}
	my ($start,$stop) = @{$self->{'selection'}};
	my ($x,$line) = $self->{'txt_ctrl'}->PositionToXY($start);
	$self->{'txt_ctrl'}->SetStyle($start,$stop,
			Wx::TextAttr->new($self->{'lines'}->[$line]->{'colour'},wxWHITE));
	$self->{'selection'} = undef;
	return 1;
}


sub OnFindText{
	my ($self,$evt) = @_;
	$self->FindFirst($evt->GetString);
}

sub GetValue{
	my ($self) = @_;
	return $self->{'value'};
}


sub Destroy{
	my ($self) = @_;
	while($self->GetSizer->Remove(0)){}
	$self->{'txt_ctrl'}->Destroy();
	$self->{'find_bar'}->Destroy();
	$self->SUPER::Destroy;
}
1;
